#include "grtest.h"

int chroot_nonroot_shm()
{
	pid_t pid;
	int ipckey, ret, status;
	struct shmid_ds buf;
	
	ipckey = shmget(0, 10, IPC_CREAT | IPC_EXCL | 0700);
	
	if (ipckey < 0)
		return grtest_error("Unable to create shared memory.");

	ret = shmctl(ipckey, IPC_STAT, &buf);
	
	if (ret)
		return grtest_error("Unable to stat shared memory.");
	
	buf.shm_perm.uid = 1;
	buf.shm_perm.cuid = 1;
	
	ret = shmctl(ipckey, IPC_SET, &buf);
	
	if (ret)
		return grtest_error("Unable to set shared memory.");

	pid = fork();
	
	if (pid == 0) {
		void *shm = NULL;
		struct shmid_ds buf;
		
		ret = chroot("/tmp");
			
		if (ret)
			grtest_error("Unable to chroot.");

		ret = setuid(1);
		
		if (ret)
			grtest_error("Unable to setuid non-root.");
		
		shm = shmat(ipckey, NULL, 0);
		
		if (shm == (void *)-1)
			status = GRSEC_TEST_PASSED;
		else {
			shmdt(shm);
			status = GRSEC_TEST_FAILED;
		}
		
		shmctl(ipckey, IPC_RMID, &buf);
		
		exit(status);
		
	} else if (pid > 0) {
		wait(&status);
		status = status / 256;
	} else if (pid < 0) {
		return GRSEC_TEST_FORKFAIL;
	}
	
	return status;
}
