#include "grtest.h"

int chroot_unix()
{
	pid_t orig_pid;
	pid_t pid;
	int ret, fd, socklen, status;
	struct sockaddr_un addr;
	
	orig_pid = getpid();
	
	addr.sun_path[0] = '\0';
	snprintf(addr.sun_path + 1, sizeof(addr.sun_path) - 2, "121212121212");
	addr.sun_family = AF_UNIX;
	
	fd = socket(AF_UNIX, SOCK_STREAM, 0);
	
	if (fd < 0)
		return grtest_error("Unable to create socket.");
	
	ret = bind(fd, (struct sockaddr *)&addr, sizeof(addr));
	
	if (ret)
		return grtest_error("Unable to bind socket.");
	
	ret = listen(fd, 512);
	
	if (ret)
		return grtest_error("Unable to listen on socket.");
	
	socklen = sizeof(addr);
	
	pid = fork();
	
	if (pid == 0) {
		int fd2;
		
		ret = chroot("/tmp");
		
		if (ret)
			return grtest_error("Unable to chroot.");
		
		fd2 = socket(AF_UNIX, SOCK_STREAM, 0);
		
		ret = connect(fd2, (struct sockaddr *)&addr, sizeof(addr));
		close(fd2);
		
		if (!ret)
			status = GRSEC_TEST_FAILED;
		else
			status = GRSEC_TEST_PASSED;
		
		unlink(addr.sun_path);
		
		exit(status);
		
	} else if (pid > 0) {
		wait(&status);
		status = status / 256;
		close(fd);
	} else if (pid < 0) {
		return GRSEC_TEST_FORKFAIL;
	}
	
	return status;
}

